#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#include <libpv.h>

static char *prompt="test";
static int shutdown = 0;

void signalhandler (int sig)
{
	if (sig == SIGPIPE) return;
	if (sig == SIGHUP) {
		fprintf(stderr, "%s SIGHUP received\n", prompt);
	}
	if (sig == SIGTERM || sig == SIGINT) {
		fprintf(stderr, "%s received SIGTERM or SIGINT, shutting down\n", prompt);
		shutdown = 1;
	}
}

int main(int argc, char *argv[])
{
	struct sigaction	action;
	pv_engine_t 		pv_engine;
	pv_engine_attr_t	attr;
	unsigned char		*pv_ucResetCounter;
	uint8_t			*pv_choke_me;
	uint16_t		*pv_kiss_me;
	char			*pv_string;
	uint16_t		*pv_array;
	char			my_string[80];
	int			err;
	int			i;

	err = pv_engine_attr_init_from_cmdline(&attr, argc, argv);
	if (err)
		exit (EXIT_FAILURE);
	err = pv_init(&pv_engine, &attr);
	if (err)
		exit (EXIT_FAILURE);

	printf("%s: engine created\n", prompt);

	/* setup signal handler */
	action.sa_handler = signalhandler;
	sigemptyset (&action.sa_mask);
	action.sa_flags = 0;
	sigaction (SIGTERM, &action, 0);	/* catches kill from the shell   */
	sigaction (SIGINT,  &action, 0);	/* catches Ctrl-C from the shell */
	sigaction (SIGPIPE, &action, 0);	/* catches Broken Pipe exception */
	sigaction (SIGHUP,  &action, 0);	/* catches HUP                   */

	pv_ucResetCounter = pv_register_uint8(&pv_engine, "ucResetCounter");
	pv_set_fn_read(&pv_engine, pv_ucResetCounter, pv_device_read_int_file_ascii);
	pv_set_fn_write(&pv_engine, pv_ucResetCounter, pv_device_write_int_file_ascii);

	pv_choke_me = pv_register_uint8(&pv_engine, "pv_choke_me");
	pv_kiss_me = pv_register_uint16(&pv_engine, "ucResetCounter");
	pv_string = pv_register_string(&pv_engine, "sampleString");
	pv_array = pv_register_uint16_array(&pv_engine, "plotArray");
	pv_default(&pv_engine, pv_string);
	printf("%s: pv_ucResetCounter=%p\n", prompt, pv_ucResetCounter);
	printf("%s: pv_choke_me=%p\n", prompt, pv_choke_me);
	printf("%s: pv_kiss_me=%p\n", prompt, pv_kiss_me);
	printf("%s: string=%s\n", prompt, pv_string);
	printf("%s: pv_array=%p\n", prompt, pv_array);

	err = pv_cmp_string(&pv_engine, pv_string, "1234");
	printf("%s: pv_cmp_string=%d\n", prompt, err);

	err = pv_set_string(&pv_engine, pv_string, "987654321");
	printf("%s: pv_set_string=%d\n", prompt, err);

	err = pv_set_string(&pv_engine, pv_string, "987654321");
	printf("%s: pv_set_string=%d\n", prompt, err);

	err = pv_get_string(&pv_engine, pv_string, my_string, sizeof(my_string));
	printf("%s: pv_get_string=%d \"%s\"\n", prompt, err, my_string);

	pv_default(&pv_engine, pv_string);
	printf("%s: string=%s\n", prompt, pv_string);

	while (!shutdown) {

		pv_lock(&pv_engine);

		/* read / modify / write */
		pv_from_device(&pv_engine, pv_ucResetCounter);
		(*pv_ucResetCounter)++;
		pv_flush(&pv_engine, pv_ucResetCounter, sizeof(*pv_ucResetCounter));
		pv_to_device(&pv_engine, pv_ucResetCounter);

		for (i=0; i < 15;i++) {
			pv_array[i] = i*(i+*pv_ucResetCounter);
		}
		pv_flush(&pv_engine, pv_array, 15*2);

		pv_unlock(&pv_engine);
		
		printf("%s: ucResetCounter=%i\n", prompt, *pv_ucResetCounter);

		printf("%s: plotArray[10]=%i\n", prompt, pv_array[10]);

		sleep(1);
		
	}
	
	pv_default(&pv_engine, pv_ucResetCounter);
	printf("%s: ucResetCounter=%i\n", prompt, *pv_ucResetCounter);

	pv_destroy(&pv_engine);
	
	return EXIT_SUCCESS;
}

